// This sketch requires the HC-05 module to start in the communication mode. No need to hold down the button
// 1. If there is any wire connected to Arduino pin 0 (RX) & 1 (TX), remove them before Step 2.
// 2. Upload this sketch and disconnect Arduino from PC.
// 3. Connect Arduino pin 0 (RX) to the HC-05 TX.
// 4. Connect Arduino pin 1 (TX) to the HC-05 RX through a voltage divider.
// 5. Pair the HC-05 module with the Bluetooth radio of your PC.

long hc05_baudrate = 38400;        // connection speed between Arduino and HC-05 in the communication mode

const long nchan = 11;  // number of reward channels
long pinout[nchan] = {2,3,4,5,6,7,8,9,10,11,12};  // digital pins assigned to the channels

void setup() {
  for (size_t m=0; m<nchan; m++)
  {
    pinMode(pinout[m], OUTPUT);
    digitalWrite(pinout[m], LOW);
  }
  pinMode(LED_BUILTIN, OUTPUT);

  Serial.begin(hc05_baudrate);
}

void loop() {
  if (0 < Serial.available()) {
    String cmd = Serial.readStringUntil('\n');
    cmd.trim();
    if (0==cmd.length()) return;

    String result = "P1";  // Command not understood
    switch (cmd.charAt(0)) {

      case 'Q':  // query current state
        result = "READY";
        break;
      
      case 'R':  // reset
        result = "OK";
        break;

      case 'Y': {  // deliver reward, Y<juice line>,<duration>,<num reward>,<pause time>
        long comma1 = cmd.indexOf(',',1);
        long comma2 = cmd.indexOf(',',comma1+1);
        long comma3 = cmd.indexOf(',',comma2+1);
        if (comma1<1 || comma2<1 | comma3<1) {
          result = "P3";  // Parameter error
          break;
        }

        unsigned long juiceline = cmd.substring(1,comma1).toInt() - 1;  // 0-based
        unsigned long duration = cmd.substring(comma1+1,comma2).toInt();
        unsigned long numreward = cmd.substring(comma2+1,comma3).toInt();
        unsigned long pausetime = cmd.substring(comma3+1).toInt();
        if (nchan<=juiceline) {
          result = "P2";  // Solenoid does not exist
          break;
        }

        for (size_t m=1; m<=numreward; m++) {
          digitalWrite(LED_BUILTIN, HIGH);
          digitalWrite(pinout[juiceline], HIGH);
          delay(duration);
          digitalWrite(pinout[juiceline], LOW);
          digitalWrite(LED_BUILTIN, LOW);
          if (m<numreward) delay(pausetime);
        }
        result = "OK";
        break;
      }
    }
    
    Serial.write((result + '\n').c_str());
  }
}
